/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { resolve } from 'node:path';
import fs from 'node:fs';
import { Messages } from '@salesforce/core';
import { ComponentSetBuilder, MetadataConverter, RegistryAccess, } from '@salesforce/source-deploy-retrieve';
import { arrayWithDeprecation, Flags, loglevel, orgApiVersionFlagWithDeprecations, SfCommand, } from '@salesforce/sf-plugins-core';
import { getPackageDirs, getSourceApiVersion } from '../../../utils/project.js';
import { SourceConvertResultFormatter } from '../../../formatters/sourceConvertResultFormatter.js';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-deploy-retrieve', 'convert.source');
export class Source extends SfCommand {
    static summary = messages.getMessage('summary');
    static description = messages.getMessage('description');
    static examples = messages.getMessages('examples');
    static requiresProject = true;
    static aliases = ['force:source:convert'];
    static deprecateAliases = true;
    static flags = {
        'api-version': {
            ...orgApiVersionFlagWithDeprecations,
            summary: messages.getMessage('flags.api-version.summary'),
        },
        loglevel,
        'root-dir': Flags.directory({
            aliases: ['rootdir'],
            deprecateAliases: true,
            char: 'r',
            summary: messages.getMessage('flags.root-dir.summary'),
            exists: true,
        }),
        'output-dir': Flags.directory({
            aliases: ['outputdir'],
            deprecateAliases: true,
            default: `metadataPackage_${Date.now()}`,
            defaultHelp: async () => Promise.resolve('metadataPackage_>timestamp<'),
            char: 'd',
            summary: messages.getMessage('flags.output-dir.summary'),
        }),
        'package-name': Flags.string({
            char: 'n',
            aliases: ['packagename'],
            deprecateAliases: true,
            summary: messages.getMessage('flags.package-name.summary'),
        }),
        manifest: Flags.file({
            char: 'x',
            summary: messages.getMessage('flags.manifest.summary'),
            description: messages.getMessage('flags.manifest.description'),
            exists: true,
        }),
        'source-dir': arrayWithDeprecation({
            char: 'p',
            aliases: ['sourcepath'],
            deprecateAliases: true,
            description: messages.getMessage('flags.source-dir.description'),
            summary: messages.getMessage('flags.source-dir.summary'),
            exclusive: ['manifest', 'metadata'],
        }),
        metadata: arrayWithDeprecation({
            char: 'm',
            summary: messages.getMessage('flags.metadata.summary'),
            exclusive: ['manifest', 'sourcepath'],
        }),
    };
    convertResult;
    flags;
    componentSet;
    async run() {
        this.flags = (await this.parse(Source)).flags;
        await this.convert();
        this.resolveSuccess();
        return this.formatResult();
    }
    async convert() {
        const paths = [];
        const { metadata, manifest } = this.flags;
        const sourcepath = this.flags['source-dir'];
        const rootdir = this.flags['root-dir'];
        if (sourcepath) {
            paths.push(...sourcepath);
        }
        // rootdir behaves exclusively to sourcepath, metadata, and manifest... to maintain backwards compatibility
        // we will check here, instead of adding the exclusive option to the flag definition so we don't break scripts
        if (rootdir && !sourcepath && !metadata && !manifest && typeof rootdir === 'string') {
            // only rootdir option passed
            paths.push(rootdir);
        }
        // no options passed, convert the default package (usually force-app)
        if (!sourcepath && !metadata && !manifest && !rootdir) {
            paths.push(this.project.getDefaultPackage().path);
        }
        this.componentSet = await ComponentSetBuilder.build({
            sourceapiversion: this.flags['api-version'] ?? (await getSourceApiVersion()),
            sourcepath: paths,
            manifest: manifest
                ? {
                    manifestPath: manifest,
                    directoryPaths: await getPackageDirs(),
                }
                : undefined,
            metadata: metadata
                ? {
                    metadataEntries: metadata,
                    directoryPaths: await getPackageDirs(),
                }
                : undefined,
            projectDir: this.project?.getPath(),
        });
        const packageName = this.flags['package-name'];
        const outputDirectory = resolve(this.flags['output-dir']);
        const registry = new RegistryAccess(undefined, this.project?.getPath());
        const converter = new MetadataConverter(registry);
        this.convertResult = await converter.convert(this.componentSet, 'metadata', {
            type: 'directory',
            outputDirectory,
            packageName,
            genUniqueDir: false,
        });
        if (packageName && this.convertResult.packagePath) {
            // SDR will build an output path like /output/directory/packageName/package.xml
            // this was breaking from toolbelt, so to revert it we copy the directory up a level and delete the original
            fs.cpSync(this.convertResult.packagePath, outputDirectory, { recursive: true });
            fs.rmSync(this.convertResult.packagePath, { recursive: true });
            this.convertResult.packagePath = outputDirectory;
        }
    }
    resolveSuccess() {
        if (!this.convertResult.packagePath) {
            process.exitCode = 1;
        }
    }
    async formatResult() {
        const formatter = new SourceConvertResultFormatter(this.convertResult);
        if (!this.jsonEnabled()) {
            formatter.display();
        }
        return formatter.getJson();
    }
}
//# sourceMappingURL=source.js.map